@@ -14,7 +14,7 @@ implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 PURPOSE.  See the GNU Library General Public License for more details.
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free Software
-Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 #ifndef _Integer_h
@@ -25,6 +25,8 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 #include <iostream.h>
 
+#undef OK
+
 struct IntRep                    // internal Integer representations
 {
   unsigned short  len;          // current length
@@ -89,45 +91,44 @@ public:
                   Integer(const Integer&);
 
                   ~Integer();
-
-  void            operator =  (const Integer&);
-  void            operator =  (long);
+  Integer&        operator =  (const Integer&);
+  Integer&        operator =  (long);
 
 // unary operations to self
 
-  void            operator ++ ();
-  void            operator -- ();
+  Integer&        operator ++ ();
+  Integer&        operator -- ();
   void            negate();          // negate in-place
   void            abs();             // absolute-value in-place
   void            complement();      // bitwise complement in-place
 
 // assignment-based operations
 
-  void            operator += (const Integer&);
-  void            operator -= (const Integer&);
-  void            operator *= (const Integer&);
-  void            operator /= (const Integer&);
-  void            operator %= (const Integer&);
-  void            operator <<=(const Integer&);
-  void            operator >>=(const Integer&);
-  void            operator &= (const Integer&);
-  void            operator |= (const Integer&);
-  void            operator ^= (const Integer&);
+  Integer&        operator += (const Integer&);
+  Integer&        operator -= (const Integer&);
+  Integer&        operator *= (const Integer&);
+  Integer&        operator /= (const Integer&);
+  Integer&        operator %= (const Integer&);
+  Integer&        operator <<=(const Integer&);
+  Integer&        operator >>=(const Integer&);
+  Integer&        operator &= (const Integer&);
+  Integer&        operator |= (const Integer&);
+  Integer&        operator ^= (const Integer&);
 
-  void            operator += (long);
-  void            operator -= (long);
-  void            operator *= (long);
-  void            operator /= (long);
-  void            operator %= (long);
-  void            operator <<=(long);
-  void            operator >>=(long);
-  void            operator &= (long);
-  void            operator |= (long);
-  void            operator ^= (long);
+  Integer&        operator += (long);
+  Integer&        operator -= (long);
+  Integer&        operator *= (long);
+  Integer&        operator /= (long);
+  Integer&        operator %= (long);
+  Integer&        operator <<=(long);
+  Integer&        operator >>=(long);
+  Integer&        operator &= (long);
+  Integer&        operator |= (long);
+  Integer&        operator ^= (long);
 
 // (constructive binary operations are inlined below)
 
-#ifdef __GNUG__
+#if defined (__GNUG__) && ! defined (__STRICT_ANSI__)
   friend Integer operator <? (const Integer& x, const Integer& y); // min
   friend Integer operator >? (const Integer& x, const Integer& y); // max
 #endif
@@ -162,9 +163,11 @@ public:
   friend void     mod(const Integer& x, const Integer& y, Integer& dest);
   friend void     divide(const Integer& x, const Integer& y, 
                          Integer& q, Integer& r);
+#ifndef __STRICT_ANSI__
   friend void     and(const Integer& x, const Integer& y, Integer& dest);
   friend void     or(const Integer& x, const Integer& y, Integer& dest);
-  friend void     xor(const Integer& x, const Integer& y, Integer& dest);
+  friend void     xor(const Integer& x, const Integer& y, Integer& dest); 
+#endif
   friend void     lshift(const Integer& x, const Integer& y, Integer& dest);
   friend void     rshift(const Integer& x, const Integer& y, Integer& dest);
   friend void     pow(const Integer& x, const Integer& y, Integer& dest);
@@ -177,9 +180,11 @@ public:
   friend void     div(const Integer& x, long y, Integer& dest);
   friend void     mod(const Integer& x, long y, Integer& dest);
   friend void     divide(const Integer& x, long y, Integer& q, long& r);
+#ifndef __STRICT_ANSI__
   friend void     and(const Integer& x, long y, Integer& dest);
   friend void     or(const Integer& x, long y, Integer& dest);
   friend void     xor(const Integer& x, long y, Integer& dest);
+#endif
   friend void     lshift(const Integer& x, long y, Integer& dest);
   friend void     rshift(const Integer& x, long y, Integer& dest);
   friend void     pow(const Integer& x, long y, Integer& dest);
@@ -189,9 +194,21 @@ public:
   friend void     add(long x, const Integer& y, Integer& dest);
   friend void     sub(long x, const Integer& y, Integer& dest);
   friend void     mul(long x, const Integer& y, Integer& dest);
+#ifndef __STRICT_ANSI__
   friend void     and(long x, const Integer& y, Integer& dest);
   friend void     or(long x, const Integer& y, Integer& dest);
   friend void     xor(long x, const Integer& y, Integer& dest);
+#endif
+
+  friend Integer  operator &  (const Integer&, const Integer&);
+  friend Integer  operator &  (const Integer&, long);
+  friend Integer  operator &  (long, const Integer&);
+  friend Integer  operator |  (const Integer&, const Integer&);
+  friend Integer  operator |  (const Integer&, long);
+  friend Integer  operator |  (long, const Integer&);
+  friend Integer  operator ^  (const Integer&, const Integer&);
+  friend Integer  operator ^  (const Integer&, long);
+  friend Integer  operator ^  (long, const Integer&);
 
 // coercion & conversion
 
@@ -249,15 +266,6 @@ public:
   Integer  operator << (const Integer&, long);
   Integer  operator >> (const Integer&, const Integer&);
   Integer  operator >> (const Integer&, long);
-  Integer  operator &  (const Integer&, const Integer&);
-  Integer  operator &  (const Integer&, long);
-  Integer  operator &  (long, const Integer&);
-  Integer  operator |  (const Integer&, const Integer&);
-  Integer  operator |  (const Integer&, long);
-  Integer  operator |  (long, const Integer&);
-  Integer  operator ^  (const Integer&, const Integer&);
-  Integer  operator ^  (const Integer&, long);
-  Integer  operator ^  (long, const Integer&);
 
   Integer  abs(const Integer&); // absolute value
   Integer  sqr(const Integer&); // square
@@ -290,14 +298,16 @@ inline Integer::Integer(const Integer&  y) :rep(Icopy(0, y.rep)) {}
 
 inline Integer::~Integer() { if (rep && !STATIC_IntRep(rep)) delete rep; }
 
-inline void  Integer::operator = (const Integer&  y)
+inline Integer&  Integer::operator = (const Integer&  y)
 {
   rep = Icopy(rep, y.rep);
+  return *this;
 }
 
-inline void Integer::operator = (long y)
+inline Integer& Integer::operator = (long y)
 {
-  rep = Icopy_long(rep, y); 
+  rep = Icopy_long(rep, y);
+  return *this;
 }
 
 inline int Integer::initialized() const
@@ -362,6 +372,7 @@ inline void  mod(const Integer& x, const Integer& y, Integer& dest)
   dest.rep = mod(x.rep, y.rep, dest.rep);
 }
 
+#ifndef __STRICT_ANSI__
 inline void  and(const Integer& x, const Integer& y, Integer& dest)
 {
   dest.rep = bitop(x.rep, y.rep, dest.rep, '&');
@@ -376,6 +387,7 @@ inline void  xor(const Integer& x, const Integer& y, Integer& dest)
 {
   dest.rep = bitop(x.rep, y.rep, dest.rep, '^');
 }
+#endif
 
 inline void  lshift(const Integer& x, const Integer& y, Integer& dest)
 {
@@ -417,6 +429,7 @@ inline void  mod(const Integer& x, long y, Integer& dest)
   dest.rep = mod(x.rep, y, dest.rep);
 }
 
+#ifndef __STRICT_ANSI__
 inline void  and(const Integer& x, long y, Integer& dest)
 {
   dest.rep = bitop(x.rep, y, dest.rep, '&');
@@ -431,6 +444,7 @@ inline void  xor(const Integer& x, long y, Integer& dest)
 {
   dest.rep = bitop(x.rep, y, dest.rep, '^');
 }
+#endif
 
 inline void  lshift(const Integer& x, long y, Integer& dest)
 {
@@ -477,6 +491,7 @@ inline void  mul(long x, const Integer& y, Integer& dest)
   dest.rep = multiply(y.rep, x, dest.rep);
 }
 
+#ifndef __STRICT_ANSI__
 inline void  and(long x, const Integer& y, Integer& dest)
 {
   dest.rep = bitop(y.rep, x, dest.rep, '&');
@@ -491,6 +506,7 @@ inline void  xor(long x, const Integer& y, Integer& dest)
 {
   dest.rep = bitop(y.rep, x, dest.rep, '^');
 }
+#endif
 
 
 // operator versions
@@ -556,116 +572,136 @@ inline int operator >= (const Integer&  x, long y)
 }
 
 
-inline void  Integer::operator += (const Integer& y)
+inline Integer&  Integer::operator += (const Integer& y)
 {
   add(*this, y, *this);
+  return *this;
 }
 
-inline void  Integer::operator += (long y)
+inline Integer&  Integer::operator += (long y)
 {
   add(*this, y, *this);
+  return *this;
 }
 
-inline void Integer::operator ++ ()
+inline Integer& Integer::operator ++ ()
 {
   add(*this, 1, *this);
+  return *this;
 }
 
 
-inline void  Integer::operator -= (const Integer& y)
+inline Integer& Integer::operator -= (const Integer& y)
 {
   sub(*this, y, *this);
+  return *this;
 }
 
-inline void  Integer::operator -= (long y)
+inline Integer& Integer::operator -= (long y)
 {
   sub(*this, y, *this);
+  return *this;
 }
 
-inline void Integer::operator -- ()
+inline Integer& Integer::operator -- ()
 {
   add(*this, -1, *this);
+  return *this;
 }
 
 
 
-inline void Integer::operator *= (const Integer& y)
+inline Integer& Integer::operator *= (const Integer& y)
 {
   mul(*this, y, *this);
+  return *this;
 }
 
-inline void Integer::operator *= (long y)
+inline Integer& Integer::operator *= (long y)
 {
   mul(*this, y, *this);
+  return *this;
 }
 
 
-inline void  Integer::operator &= (const Integer& y)
+inline Integer& Integer::operator &= (const Integer& y)
 {
-  and(*this, y, *this);
+  rep = bitop(rep, y.rep, rep, '&');
+  return *this;
 }
 
-inline void  Integer::operator &= (long y)
+inline Integer& Integer::operator &= (long y)
 {
-  and(*this, y, *this);
+  rep = bitop(rep, y, rep, '&');
+  return *this;
 }
 
-inline void  Integer::operator |= (const Integer& y)
+inline Integer& Integer::operator |= (const Integer& y)
 {
-  or(*this, y, *this);
+  rep = bitop(rep, y.rep, rep, '|');
+  return *this;
 }
 
-inline void  Integer::operator |= (long y)
+inline Integer& Integer::operator |= (long y)
 {
-  or(*this, y, *this);
+  rep = bitop(rep, y, rep, '|');
+  return *this;
 }
 
 
-inline void  Integer::operator ^= (const Integer& y)
+inline Integer& Integer::operator ^= (const Integer& y)
 {
-  xor(*this, y, *this);
+  rep = bitop(rep, y.rep, rep, '^');
+  return *this;
 }
 
-inline void  Integer::operator ^= (long y)
+inline Integer& Integer::operator ^= (long y)
 {
-  xor(*this, y, *this);
+  rep = bitop(rep, y, rep, '^');
+  return *this;
 }
 
 
 
-inline void Integer::operator /= (const Integer& y)
+inline Integer& Integer::operator /= (const Integer& y)
 {
   div(*this, y, *this);
+  return *this;
 }
 
-inline void Integer::operator /= (long y)
+inline Integer& Integer::operator /= (long y)
 {
   div(*this, y, *this);
+  return *this;
 }
 
 
-inline void Integer::operator <<= (const Integer&  y)
+inline Integer& Integer::operator <<= (const Integer&  y)
 {
   lshift(*this, y, *this);
+  return *this;
 }
 
-inline void Integer::operator <<= (long  y)
+inline Integer& Integer::operator <<= (long  y)
 {
   lshift(*this, y, *this);
+  return *this;
 }
 
 
-inline void Integer::operator >>= (const Integer&  y)
+inline Integer& Integer::operator >>= (const Integer&  y)
 {
   rshift(*this, y, *this);
+  return *this;
 }
 
-inline void  Integer::operator >>= (long y)
+inline Integer& Integer::operator >>= (long y)
 {
   rshift(*this, y, *this);
+  return *this;
 }
 
-#ifdef __GNUG__
+#if defined (__GNUG__) && ! defined (__STRICT_ANSI__)
 inline Integer operator <? (const Integer& x, const Integer& y)
 {
   return (compare(x.rep, y.rep) <= 0) ? x : y;
@@ -724,7 +760,7 @@ inline long lg(const Integer& x)
 
 // constructive operations 
 
-#if defined(__GNUG__) && !defined(NO_NRV)
+#if defined(__GNUG__) && !defined(_G_NO_NRV)
 
 inline Integer  operator +  (const Integer& x, const Integer& y) return r
 {
@@ -778,47 +814,47 @@ inline Integer sqr(const Integer& x) return r
 
 inline Integer  operator &  (const Integer& x, const Integer& y) return r
 {
-  and(x, y, r);
+  r.rep = bitop(x.rep, y.rep, r.rep, '&');
 }
 
 inline Integer  operator &  (const Integer& x, long y) return r
 {
-  and(x, y, r);
+  r.rep = bitop(x.rep, y, r.rep, '&');
 }
 
 inline Integer  operator &  (long  x, const Integer& y) return r
 {
-  and(x, y, r);
+  r.rep = bitop(y.rep, x, r.rep, '&');
 }
 
 inline Integer  operator |  (const Integer& x, const Integer& y) return r
 {
-  or(x, y, r);
+  r.rep = bitop(x.rep, y.rep, r.rep, '|');
 }
 
 inline Integer  operator |  (const Integer& x, long y) return r
 {
-  or(x, y, r);
+  r.rep = bitop(x.rep, y, r.rep, '|');
 }
 
 inline Integer  operator |  (long  x, const Integer& y) return r
 {
-  or(x, y, r);
+  r.rep = bitop(y.rep, x, r.rep, '|');
 }
 
 inline Integer  operator ^  (const Integer& x, const Integer& y) return r
 {
-  xor(x, y, r);
+  r.rep = bitop(x.rep, y.rep, r.rep, '^');
 }
 
 inline Integer  operator ^  (const Integer& x, long y) return r
 {
-  xor(x, y, r);
+  r.rep = bitop (x.rep, y, r.rep, '^');
 }
 
 inline Integer  operator ^  (long  x, const Integer& y) return r
 {
-  xor(x, y, r);
+  r.rep = bitop (y.rep, x, r.rep, '^');
 }
 
 inline Integer  operator /  (const Integer& x, const Integer& y) return r
@@ -987,17 +1023,17 @@ inline Integer  operator |  (long  x, const Integer& y)
 
 inline Integer  operator ^  (const Integer& x, const Integer& y) 
 {
-  Integer r; xor(x, y, r); return r;
+  Integer r; r.rep = bitop(x.rep, y.rep, r.rep, '^'); return r;
 }
 
 inline Integer  operator ^  (const Integer& x, long y) 
 {
-  Integer r; xor(x, y, r); return r;
+  Integer r; r.rep = bitop(x.rep, y, r.rep, '^'); return r;
 }
 
 inline Integer  operator ^  (long  x, const Integer& y) 
 {
-  Integer r; xor(x, y, r); return r;
+  Integer r; r.rep = bitop(y.rep, x, r.rep, '^'); return r;
 }
 
 inline Integer  operator /  (const Integer& x, const Integer& y) 
@@ -1084,13 +1120,15 @@ inline Integer  gcd(const Integer& x, const Integer& y)
 
 #endif  /* NO_NRV */
 
-inline void Integer::operator %= (const Integer& y)
+inline Integer& Integer::operator %= (const Integer& y)
 {
   *this = *this % y; // mod(*this, y, *this) doesn't work.
+  return *this;
 }
 
-inline void Integer::operator %= (long y)
+inline Integer& Integer::operator %= (long y)
 {
   *this = *this % y; // mod(*this, y, *this) doesn't work.
+  return *this;
 }
 #endif /* !_Integer_h */
