@@ -30,54 +30,6 @@ the executable file might be covered by the GNU General Public License. */
 #endif
 #include <string.h>
 
-void _IO_un_link( _IO_FILE *);
-void _IO_link_in( _IO_FILE *);
-_IO_size_t _IO_least_marker( register _IO_FILE * );
-void _IO_switch_to_main_get_area( _IO_FILE * );
-void _IO_switch_to_backup_area( register _IO_FILE * );
-void _IO_free_backup_area( register _IO_FILE * );
-int __overflow( _IO_FILE *, int );
-static int save_for_backup( _IO_FILE * );
-int __underflow( _IO_FILE * );
-int __uflow( _IO_FILE * );
-void _IO_setb( _IO_FILE *, char *, char *, int );
-void _IO_doallocbuf( register _IO_FILE * );
-int _IO_default_underflow( _IO_FILE * );
-int _IO_default_uflow( _IO_FILE * );
-_IO_size_t _IO_default_xsputn( register _IO_FILE *, const void *, _IO_size_t );
-_IO_size_t _IO_sgetn( _IO_FILE *, void *, _IO_size_t );
-_IO_size_t _IO_default_xsgetn( _IO_FILE *, void *, _IO_size_t );
-int _IO_sync( register _IO_FILE * );
-_IO_FILE* _IO_default_setbuf( register _IO_FILE *, char *, _IO_ssize_t );
-_IO_pos_t _IO_default_seekpos( _IO_FILE *, _IO_pos_t, int );
-int _IO_default_doallocate( _IO_FILE * );
-void _IO_init( register _IO_FILE *, int );
-int _IO_default_sync( _IO_FILE * );
-void _IO_default_finish( _IO_FILE * , int);
-_IO_pos_t _IO_default_seekoff( register _IO_FILE *, _IO_off_t, int, int );
-int _IO_sputbackc( register _IO_FILE *, int );
-int _IO_sungetc( register _IO_FILE * );
-int _IO_set_column( register _IO_FILE *, int );
-unsigned _IO_adjust_column( unsigned, const char *, int );
-int _IO_get_column( register _IO_FILE * );
-int _IO_flush_all( void );
-void _IO_flush_all_linebuffered( void );
-void _IO_unbuffer_all( void );
-void _IO_cleanup( void );
-void _IO_init_marker( struct _IO_marker *, _IO_FILE * );
-void _IO_remove_marker( register struct _IO_marker * );
-int _IO_marker_difference( struct _IO_marker *, struct _IO_marker * );
-int _IO_marker_delta( struct _IO_marker * );
-int _IO_seekmark( _IO_FILE *, struct _IO_marker *, int );
-void _IO_unsave_markers( register _IO_FILE * );
-int _IO_nobackup_pbackfail( register _IO_FILE *, int );
-int _IO_default_pbackfail( register _IO_FILE *, int );
-_IO_pos_t _IO_default_seek( _IO_FILE *, _IO_off_t, int );
-int _IO_default_stat( _IO_FILE *, void * );
-_IO_ssize_t _IO_default_read( register _IO_FILE *, void *, _IO_ssize_t );
-_IO_ssize_t _IO_default_write( register _IO_FILE *, const void *,
-										_IO_ssize_t );
-
 void
 DEFUN(_IO_un_link, (fp),
       _IO_FILE *fp)
@@ -326,7 +278,7 @@ DEFUN(_IO_setb, (f, b, eb, a),
       _IO_FILE *f AND char *b AND char *eb AND int a)
 {
   if (f->_IO_buf_base && !(f->_flags & _IO_USER_BUF))
-    FREE_BUF(f->_IO_buf_base, _IO_blen(f));
+    FREE_BUF(f->_IO_buf_base);
   f->_IO_buf_base = b;
   f->_IO_buf_end = eb;
   if (a)
@@ -486,9 +438,9 @@ int
 DEFUN(_IO_default_doallocate, (fp),
       _IO_FILE *fp)
 {
-  char *buf;
-
-  ALLOC_BUF(buf, _IO_BUFSIZ, EOF);
+  char *buf = ALLOC_BUF(_IO_BUFSIZ);
+  if (buf == NULL)
+    return EOF;
   _IO_setb(fp, buf, buf+_IO_BUFSIZ, 1);
   return 1;
 }
@@ -526,13 +478,13 @@ DEFUN(_IO_default_sync, (fp),
    current implementation, this function can get called twice! */
 
 void
-DEFUN(_IO_default_finish, (fp, dummy),
-      _IO_FILE *fp AND int dummy)
+DEFUN(_IO_default_finish, (fp),
+      _IO_FILE *fp)
 {
   struct _IO_marker *mark;
   if (fp->_IO_buf_base && !(fp->_flags & _IO_USER_BUF))
     {
-      FREE_BUF(fp->_IO_buf_base, _IO_blen(fp));
+      FREE_BUF(fp->_IO_buf_base);
       fp->_IO_buf_base = fp->_IO_buf_end = NULL;
     }
 
@@ -607,7 +559,6 @@ DEFUN(_IO_set_column, (fp, c),
     fp->_column = c - (fp->_IO_write_ptr - fp->_IO_write_base);
 }
 #else
-
 int
 DEFUN(_IO_set_column, (fp, i),
       register _IO_FILE *fp AND int i)
@@ -645,7 +596,6 @@ DEFUN_VOID(_IO_flush_all)
 {
   int result = 0;
   _IO_FILE *fp;
-
   for (fp = _IO_list_all; fp != NULL; fp = fp->_chain)
     if (fp->_IO_write_ptr > fp->_IO_write_base
 	&& _IO_OVERFLOW (fp, EOF) == EOF)
@@ -657,9 +607,8 @@ void
 DEFUN_VOID(_IO_flush_all_linebuffered)
 {
   _IO_FILE *fp;
-
   for (fp = _IO_list_all; fp != NULL; fp = fp->_chain)
-    if ((fp->_flags & _IO_NO_WRITES) == 0 && fp->_flags & _IO_LINE_BUF)
+    if (fp->_flags & _IO_LINE_BUF)
       _IO_OVERFLOW (fp, EOF);
 }
 
@@ -667,7 +616,6 @@ void
 DEFUN_VOID(_IO_unbuffer_all)
 {
   _IO_FILE *fp;
-
   for (fp = _IO_list_all; fp != NULL; fp = fp->_chain)
     if (! (fp->_flags & _IO_UNBUFFERED))
       _IO_SETBUF (fp, NULL, 0);
@@ -902,115 +850,3 @@ __io_defs io_defs__;
 #endif
 
 #endif /* TODO */
-
-#ifdef _IO_USE_LIBC
-/* We only include those files which are not compatible with
-   libio in libc. We need to make sure our version is always
-   used during linking. */
-extern int vfprintf ();
-extern _IO_FILE * popen ();
-static const void * const __dummy [] =
-{
-  &_IO_file_doallocate,
-  &_IO_dtoa,
-  &_IO_strtod,
-  &_IO_adjust_column,
-  &_IO_cleanup,
-  &_IO_default_doallocate,
-  &_IO_default_finish,
-  &_IO_default_pbackfail,
-  &_IO_default_read,
-  &_IO_default_seek,
-  &_IO_default_seekoff,
-  &_IO_default_seekpos,
-  &_IO_default_setbuf,
-  &_IO_default_stat,
-  &_IO_default_sync,
-  &_IO_default_uflow,
-  &_IO_default_underflow,
-  &_IO_default_write,
-  &_IO_default_xsgetn,
-  &_IO_default_xsputn,
-  &_IO_doallocbuf,
-  &_IO_flush_all,
-  &_IO_flush_all_linebuffered,
-  &_IO_free_backup_area,
-  &_IO_get_column,
-  &_IO_init,
-  &_IO_init_marker,
-  &_IO_least_marker,
-  &_IO_link_in,
-  &_IO_marker_delta,
-  &_IO_marker_difference,
-  &_IO_nobackup_pbackfail,
-  &_IO_remove_marker,
-  &_IO_seekmark,
-  &_IO_set_column,
-  &_IO_setb,
-  &_IO_sgetn,
-  &_IO_sputbackc,
-  &_IO_sungetc,
-  &_IO_switch_to_backup_area,
-  &_IO_switch_to_get_mode,
-  &_IO_switch_to_main_get_area,
-  &_IO_sync,
-  &_IO_un_link,
-  &_IO_unbuffer_all,
-  &_IO_unsave_markers,
-  &__overflow,
-  &__uflow,
-  &__underflow,
-  &_IO_do_write,
-  &_IO_file_attach,
-  &_IO_file_close,
-  &_IO_file_close_it,
-  &_IO_file_finish,
-  &_IO_file_fopen,
-  &_IO_file_init,
-  &_IO_file_jumps,
-  &_IO_file_overflow,
-  &_IO_file_read,
-  &_IO_file_seek,
-  &_IO_file_seekoff,
-  &_IO_file_setbuf,
-  &_IO_file_stat,
-  &_IO_file_sync,
-  &_IO_file_underflow,
-  &_IO_file_write,
-  &_IO_file_xsputn,
-  &_IO_vfprintf,
-  &vfprintf,
-  &_IO_vfscanf,
-  &_IO_padn,
-  &_IO_seekoff,
-  &_IO_seekpos,
-  &_IO_outfloat,
-  &_IO_str_count,
-  &_IO_str_finish,
-  &_IO_str_init_readonly,
-  &_IO_str_init_static,
-  &_IO_str_jumps,
-  &_IO_str_overflow,
-  &_IO_str_pbackfail,
-  &_IO_str_seekoff,
-  &_IO_str_underflow,
-  &_IO_fclose,
-  &_IO_popen,
-  &popen,
-  &_IO_proc_close,
-  &_IO_proc_jumps,
-  &_IO_proc_open,
-  &_IO_fdopen,
-  &_IO_fflush,
-  &_IO_fopen,
-  &_IO_fputs,
-  &_IO_fwrite,
-  &_IO_puts,
-  &_IO_setbuffer,
-  &_IO_setvbuf,
-  &_IO_vsprintf,
-  &_IO_vsscanf,
-  &_IO_vasprintf,
-  &_IO_vsnprintf
-};
-#endif
