@@ -36,7 +36,3 @@ DEFUN(_IO_puts, (str),
     return EOF;
   return len+1;
 }
-
-#if defined(__ELF__) && defined(_IO_USE_LIBC)
-#pragma weak puts = _IO_puts
-#endif
