@@ -13,8 +13,8 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU CC; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+along with this library; see the file COPYING.  If not, write to the Free
+Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 As a special exception, if you link this library with files
 compiled with a GNU compiler to produce an executable, this does not cause
@@ -29,17 +29,14 @@ the executable file might be covered by the GNU General Public License. */
 #define _IONBF 2 /* No buffering. */
 
 int
-_IO_setvbuf(fp, buf, mode, size)
-     _IO_FILE* fp;
-     char* buf;
-     int mode;
-     _IO_size_t size;
+DEFUN(_IO_setvbuf, (fp, buf, mode, size),
+      _IO_FILE* fp AND char* buf AND int mode AND _IO_size_t size)
 {
   CHECK_FILE(fp, EOF);
   switch (mode)
     {
     case _IOFBF:
-      fp->_IO_file_flags &= ~_IO_LINE_BUF;
+      fp->_IO_file_flags &= ~_IO_LINE_BUF|_IO_UNBUFFERED;
       if (buf == NULL)
 	{
 	  if (fp->_IO_buf_base == NULL)
@@ -57,21 +54,25 @@ _IO_setvbuf(fp, buf, mode, size)
 
 		 A possibly cleaner alternative would be to add an
 		 extra flag, but then flags are a finite resource.  */
-	      if (fp->_jumps->__doallocate (fp) < 0)
+	      if (_IO_DOALLOCATE (fp) < 0)
 		return EOF;
 	      fp->_IO_file_flags &= ~_IO_LINE_BUF;
 	    }
 	  return 0;
 	}
-      return fp->_jumps->__setbuf(fp, buf, size);
+      break;
     case _IOLBF:
+      fp->_IO_file_flags &= ~_IO_UNBUFFERED;
       fp->_IO_file_flags |= _IO_LINE_BUF;
       if (buf == NULL)
 	return 0;
-      return fp->_jumps->__setbuf(fp, buf, size);
+      break;
     case _IONBF:
-      return fp->_jumps->__setbuf(fp, NULL, 0);
+      buf = NULL;
+      size = 0;
+      break;
     default:
       return EOF;
     }
+  return _IO_SETBUF (fp, buf, size) == NULL ? EOF : 0;
 }
