@@ -29,7 +29,7 @@ t2 ()
   SCAN ("test ", "%*s%n",  n);
   SCAN ("test ",   "%2*s%n",  n);
   SCAN ("12 ",   "%l2d",  n);
-  SCAN ("12 ",   "%2ld",  n);
+  SCAN ("12 ",   "%2ld",  N);
 }
 
 int
