@@ -2,6 +2,5 @@
 // Copyright (C) 1994 Free Software Foundation
 
 #ifndef __COMPLEX_H__
-#define __COMPLEX_H__
-#include <complex>
+#include <std/complex.h>
 #endif
